#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Models for storing inactive and deprecated tables from the CAL-ACCESS database.
"""
from __future__ import unicode_literals

# Models
from django.db import models
from calaccess_raw import fields
from .base import CalAccessBaseModel
from django.utils.encoding import python_2_unicode_compatible

# Annotations
from calaccess_raw import annotations
from calaccess_raw.annotations import DocumentCloud


@python_2_unicode_compatible
class BallotMeasuresCd(CalAccessBaseModel):
    """
    Ballot-measure dates and times.
    """
    UNIQUE_KEY = "FILER_ID"
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=7),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=19),
    ]
    election_date = fields.DateField(
        db_column='ELECTION_DATE',
        null=True,
        help_text="Ballot measure election date"
    )
    filer_id = fields.IntegerField(
        verbose_name='filer ID',
        db_column='FILER_ID',
        null=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    measure_no = fields.CharField(
        db_column='MEASURE_NO',
        max_length=2,
        help_text="Ballot measure number"
    )
    measure_name = fields.CharField(
        db_column='MEASURE_NAME',
        max_length=163,
        help_text="Ballot measure full name"
    )
    measure_short_name = fields.CharField(
        db_column='MEASURE_SHORT_NAME',
        max_length=50,
        blank=True,
        help_text="Ballot measure short name"
    )
    jurisdiction = fields.CharField(
        db_column='JURISDICTION',
        max_length=9,
        help_text="This field is undocumented"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'BALLOT_MEASURES_CD'
        ordering = (
            "-election_date",
            "measure_no",
            "measure_short_name",
            "measure_name"
        )

    def __str__(self):
        return self.measure_name


@python_2_unicode_compatible
class CvrF470Cd(CalAccessBaseModel):
    """
    The cover page for officeholder and candidate short and supplemental forms.
    """
    UNIQUE_KEY = (
        "FILING_ID",
        "AMEND_ID",
        "REC_TYPE",
        "FORM_TYPE",
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=8),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=30, end_page=32),
        DocumentCloud(id='2711616-MapCalFormat2Fields', start_page=15, end_page=16),
        DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
        DocumentCloud(id='2712034-Cal-Format-201', start_page=29, end_page=30),
    ]
    FILING_FORMS = [
        annotations.get_form('F470'),
    ]
    amend_id = fields.IntegerField(
        db_column="AMEND_ID",
        db_index=True,
        help_text="Amendment Identification number. A number of 0 is an original filing and 1 "
                  "to 999 amendments."
    )
    cand_adr1 = fields.CharField(
        db_column="CAND_ADR1",
        blank=True,
        max_length=55,
        help_text="First line of the filer's street address."
    )
    cand_adr2 = fields.CharField(
        db_column="CAND_ADR2",
        blank=True,
        max_length=55,
        help_text="Second line of the filer's street address. "
    )
    cand_city = fields.CharField(
        db_column="CAND_CITY",
        blank=True,
        max_length=30,
        help_text="Candidate/Officeholder's City."
    )
    cand_email = fields.CharField(
        db_column="CAND_EMAIL",
        blank=True,
        max_length=60,
        help_text="Candidate/Officeholder's EMail address. Not required by the form."
    )
    cand_fax = fields.CharField(
        db_column="CAND_FAX",
        blank=True,
        max_length=20,
        help_text="Candidate/Officeholder's FAX Phone Number. Not required by the form."
    )
    cand_phon = fields.CharField(
        db_column="CAND_PHON",
        blank=True,
        max_length=20,
        help_text="Candidate/Officeholder's phone number."
    )
    cand_st = fields.CharField(
        db_column="CAND_ST",
        blank=True,
        max_length=2,
        help_text="Filer's State"
    )
    cand_zip4 = fields.CharField(
        db_column="CAND_ZIP4",
        blank=True,
        max_length=10,
        help_text="Filer's zipcode"
    )
    date_1000 = fields.DateField(
        db_column="DATE_1000",
        help_text="Date contributions totaling $1,000 or more. (For the 470-S)"
    )
    dist_no = fields.CharField(
        db_column="DIST_NO",
        blank=True,
        max_length=3,
        help_text="District number for the office being sought. Populated for Senate, Assembly, "
                  "or Board of Equalization races."
    )
    elect_date = fields.DateField(
        db_column="ELECT_DATE",
        help_text="Date of the general election. Required for filings in even years."
    )
    ENTITY_CD_CHOICES = (
        ('CAO', annotations.choices.CAMPAIGN_ENTITY_CODES['CAO']),
    )
    entity_cd = fields.CharField(
        db_column="ENTITY_CD",
        blank=True,
        choices=ENTITY_CD_CHOICES,
        max_length=3,
        help_text="The filer's entity code. The value of this column will always be "
                  "Candidate/Office Holder (CAO) for this table.",
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=29)
        ]
    )
    filer_id = fields.CharField(
        db_column="FILER_ID",
        blank=True,
        max_length=9,
        help_text="Filer's unique identification number."
    )
    filer_namf = fields.CharField(
        db_column="FILER_NAMF",
        blank=True,
        max_length=45,
        help_text="Filer's First Name(s) - required for individuals"
    )
    filer_naml = fields.CharField(
        db_column="FILER_NAML",
        blank=True,
        max_length=200,
        help_text="Filer's Last Name/Committee name"
    )
    filer_nams = fields.CharField(
        db_column="FILER_NAMS",
        blank=True,
        max_length=10,
        help_text="Filer's Name Suffix"
    )
    filer_namt = fields.CharField(
        db_column="FILER_NAMT",
        blank=True,
        max_length=10,
        help_text="The filer's prefix or title that preceeds their name if they are an individual."
    )
    filing_id = fields.IntegerField(
        db_column="FILING_ID",
        db_index=True,
        help_text="Unique filing identification number."
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS])
    form_type = fields.CharField(
        db_column="FORM_TYPE",
        choices=FORM_TYPE_CHOICES,
        db_index=True,
        max_length=4,
        help_text="Type of Filing or Formset. The value of this column will always "
                  "be equal to F470.",
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=29),
        ]
    )
    JURIS_CD_CHOICES = annotations.sort_choices(annotations.choices.JURIS_CODES)
    juris_cd = fields.CharField(
        db_column="JURIS_CD",
        choices=JURIS_CD_CHOICES,
        blank=True,
        max_length=3,
        help_text="Office Jurisdiction Code",
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=29),
        ]
    )
    juris_dscr = fields.CharField(
        db_column="JURIS_DSCR",
        blank=True,
        max_length=40,
        help_text="Office jurisdiction description text reqired if the jurisdiction code "
                  "(Juris_cd) is equal to CIT, CTY, LOC, or OTH."
    )
    OFF_S_H_CD_CHOICES = annotations.sort_choices(annotations.choices.OFF_S_H_CODES)
    off_s_h_cd = fields.CharField(
        db_column="OFF_S_H_CD",
        choices=OFF_S_H_CD_CHOICES,
        blank=True,
        max_length=1,
        help_text='Office Sought/Held code. Legal values are "S" for sought and "H" for held.',
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=30),
        ]
    )
    offic_dscr = fields.CharField(
        db_column="OFFIC_DSCR",
        blank=True,
        max_length=40,
        help_text="Office sought description used if the office code is other (OTH)."
    )
    OFFICE_CD_CODES = annotations.sort_choices(annotations.choices.OFFICE_CODES)
    office_cd = fields.CharField(
        db_column="OFFICE_CD",
        choices=OFFICE_CD_CODES,
        blank=True,
        max_length=3,
        help_text="Code that identifies the office being sought. See the CAL document for "
                  "a list of valid codes.",
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=29),
        ]
    )
    REC_TYPE_CHOICES = (
        ('CVR', 'Cover Page'),
    )
    rec_type = fields.CharField(
        db_column="REC_TYPE",
        choices=REC_TYPE_CHOICES,
        blank=True,
        max_length=3,
        help_text="Type of CAL record. This column will always contain CVR.",
        documentcloud_pages=[
            DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=22),
            DocumentCloud(id='2712034-Cal-Format-201', start_page=29),
        ]
    )
    report_num = fields.CharField(
        db_column="REPORT_NUM",
        blank=True,
        max_length=3,
        help_text="Report Number; 000 Original; 001-999 Amended as reported in the filing."
    )
    rpt_date = fields.DateField(
        db_column="RPT_DATE",
        db_index=True,
        null=True,
        help_text="Date this report is filed as reported by the filer."
    )

    def __str__(self):
        return str(self.amend_id)

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'CVR_F470_CD'


@python_2_unicode_compatible
class FilerTypePeriodsCd(CalAccessBaseModel):
    """
    Undocumented.

    The table's official description contains this note: "J M needs to document. This is
    in his list of tables designed for future enhancements."
    """
    UNIQUE_KEY = (
        "ELECTION_TYPE",
        "FILER_TYPE",
        "PERIOD_ID",
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=8),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=71),
    ]
    ELECTION_TYPE_CHOICES = (
        (0, 'N/A'),
        (3001, 'GENERAL'),
        (3002, 'PRIMARY'),
        (3003, 'RECALL'),
        (3004, 'SPECIAL ELECTION'),
        (3005, 'OFFICEHOLDER'),
        (3006, 'SPECIAL RUNOFF'),
    )
    election_type = fields.IntegerField(
        db_column="ELECTION_TYPE",
        db_index=True,
        choices=ELECTION_TYPE_CHOICES,
        help_text="Election type",
        documentcloud_pages=[
            DocumentCloud(id='2774529-Lookup-Codes-Cd', start_page=3, end_page=4),
        ],
    )
    filer_type = fields.ForeignKeyField(
        'FilerTypesCd',
        related_name='filing_type_periods',
        db_constraint=False,
        db_column="FILER_TYPE",
        db_index=True,
        help_text="Foreign key referencing FilerTypesCd.filer_type",
        on_delete=models.CASCADE
    )
    period_id = fields.ForeignKeyField(
        'FilingPeriodCd',
        related_name='filing_type_periods',
        db_constraint=False,
        db_column="PERIOD_ID",
        db_index=True,
        help_text="Foreign key referencing FilingPeriodCd.period_id",
        on_delete=models.CASCADE
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'FILER_TYPE_PERIODS_CD'

    def __str__(self):
        return str(self.election_type)


@python_2_unicode_compatible
class LobbyistContributions1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of campaign contributions made by lobbyists.

    This table is 95 percent identical to LOBBYIST_CONTRIBUTIONS2_CD and LOBBYIST_CONTRIBUTIONS3_CD.

    According to "Cal-Access Tables, Columns, Indexes", this is a temporary
    table used to generate the actual Lobbyist contribution disclosure table,
    which is LOBBYIST_CONTRIBUTIONS3_CD.

    Also, the most recent values observed in FILING_PERIOD_START_DT are for the
    April 2001, so probably this table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id="2711614-CalAccessTablesWeb", start_page=10),
        DocumentCloud(id="2711614-CalAccessTablesWeb", start_page=92, end_page=93),
    ]
    filer_id = fields.IntegerField(
        verbose_name='filer ID',
        db_column='FILER_ID',
        null=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    filing_period_start_dt = fields.DateField(
        null=True,
        db_column='FILING_PERIOD_START_DT',
        verbose_name='Filing period start date',
        help_text='Start date of filing period',
    )
    filing_period_end_dt = fields.DateField(
        db_column='FILING_PERIOD_END_DT',
        null=True,
        verbose_name='Filing period end date',
        help_text='End date of filing period',
    )
    contribution_dt = fields.CharField(
        db_column='CONTRIBUTION_DT',
        max_length=32,
        blank=True,
        verbose_name='Contribution date',
        help_text='Date of contribution',
    )
    recipient_name = fields.CharField(
        db_column='RECIPIENT_NAME',
        max_length=106,
        blank=True,
        help_text="Recipient's name"
    )
    recipient_id = fields.IntegerField(
        db_column='RECIPIENT_ID',
        blank=True,
        null=True,
        help_text="Recipient's identification number"
    )
    amount = fields.FloatField(
        db_column='AMOUNT',
        blank=True,
        null=True,
        help_text="Amount received"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_CONTRIBUTIONS1_CD'
        ordering = ("-filing_period_start_dt",)

    def __str__(self):
        return str(self.filer_id)


@python_2_unicode_compatible
class LobbyistContributions2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of campaign contributions made by lobbyists.

    This table is 95 percent identical to LOBBYIST_CONTRIBUTIONS1_CD and LOBBYIST_CONTRIBUTIONS3_CD.

    According to "Cal-Access Tables, Columns, Indexes", this is a temporary
    table used to generate the actual Lobbyist contribution disclosure table,
    which is LOBBYIST_CONTRIBUTIONS3_CD.

    Also, the most recent values observed in FILING_PERIOD_START_DT are for the
    April 2001, so probably this table is no longer in use.
    """
    UNIQUE_KEY = False
    filer_id = fields.IntegerField(
        verbose_name='filer ID',
        db_column='FILER_ID',
        null=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=10, end_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=93, end_page=94),
    ]
    filing_period_start_dt = fields.DateField(
        null=True,
        db_column='FILING_PERIOD_START_DT',
        verbose_name='Filing period start date',
        help_text='Start date of filing period',
    )
    filing_period_end_dt = fields.DateField(
        db_column='FILING_PERIOD_END_DT',
        null=True,
        verbose_name='Filing period end date',
        help_text='End date of filing period',
    )
    contribution_dt = fields.CharField(
        db_column='CONTRIBUTION_DT',
        max_length=32,
        blank=True,
        verbose_name='Contribution date',
        help_text='Date of contribution',
    )
    recipient_name = fields.CharField(
        db_column='RECIPIENT_NAME',
        max_length=106,
        blank=True,
        help_text="Recipient's name"
    )
    recipient_id = fields.IntegerField(
        db_column='RECIPIENT_ID',
        blank=True,
        null=True,
        help_text="Recipient's identification number"
    )
    amount = fields.FloatField(
        db_column='AMOUNT',
        blank=True,
        null=True,
        help_text="Amount received"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_CONTRIBUTIONS2_CD'
        ordering = ("-filing_period_start_dt",)

    def __str__(self):
        return str(self.filer_id)


@python_2_unicode_compatible
class LobbyistContributions3Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of campaign contributions made by lobbyists.

    This table is 95 percent identical to LOBBYIST_CONTRIBUTIONS1_CD and LOBBYIST_CONTRIBUTIONS2_CD.

    According to "Cal-Access Tables, Columns, Indexes", this is the actual
    Lobbyist contribution disclosure table generated from the other two
    temporary tables: LOBBYIST_CONTRIBUTIONS1_CD and LOBBYIST_CONTRIBUTIONS2_CD.

    Also, the most recent values observed in FILING_PERIOD_START_DT are for the
    April 2001, so probably this table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=94),
    ]
    filer_id = fields.IntegerField(
        verbose_name='filer ID',
        db_column='FILER_ID',
        null=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    filing_period_start_dt = fields.DateField(
        null=True,
        db_column='FILING_PERIOD_START_DT',
        verbose_name='Filing period start date',
        help_text='Start date of filing period',
    )
    filing_period_end_dt = fields.DateField(
        db_column='FILING_PERIOD_END_DT',
        null=True,
        verbose_name='Filing period end date',
        help_text='End date of filing period',
    )
    contribution_dt = fields.CharField(
        db_column='CONTRIBUTION_DT',
        max_length=32,
        blank=True,
        verbose_name='Contribution date',
        help_text='Date of contribution',
    )
    recipient_name = fields.CharField(
        db_column='RECIPIENT_NAME',
        max_length=106,
        blank=True,
        help_text="Recipient's name"
    )
    recipient_id = fields.IntegerField(
        db_column='RECIPIENT_ID',
        blank=True,
        null=True,
        help_text="Recipient's identification number"
    )
    amount = fields.FloatField(
        db_column='AMOUNT',
        blank=True,
        null=True,
        help_text="Amount received"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_CONTRIBUTIONS3_CD'
        ordering = ("-filing_period_start_dt",)

    def __str__(self):
        return str(self.filer_id)


@python_2_unicode_compatible
class LobbyistEmpLobbyist1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_EMP_LOBBYIST2_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the distinct SESSION_ID values span from 1995 to 2001, so probably this
    table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=94, end_page=95),
    ]
    lobbyist_id = fields.IntegerField(
        db_column='LOBBYIST_ID',
        verbose_name="Lobbyist ID",
        help_text="Lobbyist identification number"
    )
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    lobbyist_last_name = fields.CharField(
        db_column='LOBBYIST_LAST_NAME',
        max_length=17,
        help_text="Lobbyist last name"
    )
    lobbyist_first_name = fields.CharField(
        db_column='LOBBYIST_FIRST_NAME',
        max_length=17,
        help_text="Lobbyist first name"
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMP_LOBBYIST1_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.lobbyist_id)


@python_2_unicode_compatible
class LobbyistEmpLobbyist2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_EMP_LOBBYIST1_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the distinct SESSION_ID values span from 1995 to 2001, so probably this
    table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=95),
    ]
    lobbyist_id = fields.IntegerField(
        db_column='LOBBYIST_ID',
        verbose_name="Lobbyist ID",
        help_text="Lobbyist identification number"
    )
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    lobbyist_last_name = fields.CharField(
        db_column='LOBBYIST_LAST_NAME',
        max_length=17,
        help_text="Lobbyist last name"
    )
    lobbyist_first_name = fields.CharField(
        db_column='LOBBYIST_FIRST_NAME',
        max_length=17,
        help_text="Lobbyist first name"
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMP_LOBBYIST2_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.lobbyist_id)


@python_2_unicode_compatible
class LobbyistEmployer1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is 99 percent identical to LOBBYIST_EMPLOYER2_CD and LOBBYIST_EMPLOYER3_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the only value in observed in SESSION_YR_1 is 1999 and the only value
    observed in SESSION_YR_2 is 2000.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=97, end_page=98),
    ]
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    INTEREST_CD_CHOICES = (
        (40301, 'AGRICULTURE'),
        (40302, 'EDUCATION'),
        (40303, 'ENTERTAINMENT/RECREATION'),
        (40304, 'FINANCE/INSURANCE'),
        (40305, 'GOVERNMENT'),
        (40306, 'HEALTH'),
        (40307, 'LABOR UNIONS'),
        (40308, 'LEGAL'),
        (40309, 'LODGING/RESTAURANTS'),
        (40310, 'MANUFACTURING/INDUSTRIAL'),
        (40311, 'MERCHANDISE/RETAIL'),
        (40312, 'MISCELLANEOUS'),
        (40313, 'OIL AND GAS'),
        (40314, 'POLITICAL ORGANIZATIONS'),
        (40315, 'PROFESSIONAL/TRADE'),
        (40316, 'PUBLIC EMPLOYEES'),
        (40317, 'REAL ESTATE'),
        (40318, 'TRANSPORTATION'),
        (40319, 'UTILITIES'),
    )
    interest_cd = fields.IntegerField(
        db_column='INTEREST_CD',
        choices=INTEREST_CD_CHOICES,
        blank=True,
        null=True,
        verbose_name="interest code",
        help_text='Interest Code',
        documentcloud_pages=[
            DocumentCloud(id='2774529-Lookup-Codes-Cd', start_page=19),
        ],
    )
    interest_name = fields.CharField(
        db_column='INTEREST_NAME',
        max_length=24,
        blank=True,
        verbose_name="Interest name",
        help_text="Interest name",
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        verbose_name="Total amount of year 1 of the session",
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        verbose_name="Total amount of year 2 of the session",
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER1_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.employer_id)


@python_2_unicode_compatible
class LobbyistEmployer2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is 99 percent identical to LOBBYIST_EMPLOYER1_CD and LOBBYIST_EMPLOYER3_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the only value in observed in SESSION_YR_1 is 1999 and the only value
    observed in SESSION_YR_2 is 2000.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=98, end_page=99),
    ]
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    INTEREST_CD_CHOICES = (
        (40301, 'AGRICULTURE'),
        (40302, 'EDUCATION'),
        (40303, 'ENTERTAINMENT/RECREATION'),
        (40304, 'FINANCE/INSURANCE'),
        (40305, 'GOVERNMENT'),
        (40306, 'HEALTH'),
        (40307, 'LABOR UNIONS'),
        (40308, 'LEGAL'),
        (40309, 'LODGING/RESTAURANTS'),
        (40310, 'MANUFACTURING/INDUSTRIAL'),
        (40311, 'MERCHANDISE/RETAIL'),
        (40312, 'MISCELLANEOUS'),
        (40313, 'OIL AND GAS'),
        (40314, 'POLITICAL ORGANIZATIONS'),
        (40315, 'PROFESSIONAL/TRADE'),
        (40316, 'PUBLIC EMPLOYEES'),
        (40317, 'REAL ESTATE'),
        (40318, 'TRANSPORTATION'),
        (40319, 'UTILITIES'),
    )
    interest_cd = fields.IntegerField(
        db_column='INTEREST_CD',
        blank=True,
        null=True,
        choices=INTEREST_CD_CHOICES,
        verbose_name="interest code",
        help_text='Interest Code',
        documentcloud_pages=[
            DocumentCloud(id='2774529-Lookup-Codes-Cd', start_page=19),
        ],
    )
    interest_name = fields.CharField(
        db_column='INTEREST_NAME',
        max_length=24,
        blank=True,
        help_text="Interest name"
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        verbose_name="Total amount of year 1 of the session",
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        verbose_name="Total amount of year 2 of the session",
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter 1 total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter 2 total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter 3 total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter 4 total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter 5 total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter 6 total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter 7 total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter 8 total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER2_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.employer_id)


@python_2_unicode_compatible
class LobbyistEmployer3Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is 99 percent identical to LOBBYIST_EMPLOYER1_CD and LOBBYIST_EMPLOYER2_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the only value in observed in SESSION_YR_1 is 1999 and the only value
    observed in SESSION_YR_2 is 2000.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=99),
    ]
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    INTEREST_CD_CHOICES = (
        (40301, 'AGRICULTURE'),
        (40302, 'EDUCATION'),
        (40303, 'ENTERTAINMENT/RECREATION'),
        (40304, 'FINANCE/INSURANCE'),
        (40305, 'GOVERNMENT'),
        (40306, 'HEALTH'),
        (40307, 'LABOR UNIONS'),
        (40308, 'LEGAL'),
        (40309, 'LODGING/RESTAURANTS'),
        (40310, 'MANUFACTURING/INDUSTRIAL'),
        (40311, 'MERCHANDISE/RETAIL'),
        (40312, 'MISCELLANEOUS'),
        (40313, 'OIL AND GAS'),
        (40314, 'POLITICAL ORGANIZATIONS'),
        (40315, 'PROFESSIONAL/TRADE'),
        (40316, 'PUBLIC EMPLOYEES'),
        (40317, 'REAL ESTATE'),
        (40318, 'TRANSPORTATION'),
        (40319, 'UTILITIES'),
    )
    interest_cd = fields.IntegerField(
        db_column='INTEREST_CD',
        blank=True,
        null=True,
        choices=INTEREST_CD_CHOICES,
        verbose_name="interest code",
        help_text='Interest Code',
        documentcloud_pages=[
            DocumentCloud(id='2774529-Lookup-Codes-Cd', start_page=19),
        ],
    )
    interest_name = fields.CharField(
        db_column='INTEREST_NAME',
        max_length=24,
        blank=True,
        help_text="Interest name"
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        verbose_name="Total amount of year 1 of the session",
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        verbose_name="Total amount of year 2 of the session",
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER3_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.employer_id)


@python_2_unicode_compatible
class LobbyistEmployerFirms1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_EMPLOYER_FIRMS2_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also the distinct SESSION_ID values span from 1995 to 2001, so probably this
    table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=95, end_page=96),
    ]
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    termination_dt = fields.CharField(
        verbose_name='termination date',
        db_column='TERMINATION_DT',
        max_length=32,
        blank=True,
        help_text="Termination effective date"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER_FIRMS1_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.employer_id)


@python_2_unicode_compatible
class LobbyistEmployerFirms2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_EMPLOYER_FIRMS1_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also the distinct SESSION_ID values span from 1995 to 2001, so probably this
    table is no longer in use.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=96),
    ]
    employer_id = fields.IntegerField(
        db_column='EMPLOYER_ID',
        help_text="Employer identification number",
        verbose_name="Employer ID"
    )
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    termination_dt = fields.CharField(
        verbose_name='termination date',
        db_column='TERMINATION_DT',
        max_length=32,
        blank=True,
        help_text="Termination effective date"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER_FIRMS2_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.employer_id)


@python_2_unicode_compatible
class LobbyistEmployerHistoryCd(CalAccessBaseModel):
    """
    Undocumented.

    An empty file of the same name is included in the Secretary of State's daily CAL-ACCESS database exports.

    This table is documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation
    on these tables. Cox 5/11/2000".

    Also, the columns on this table are identical to the columns on the
    LOBBYIST_EMPLOYER1_CD, LOBBYIST_EMPLOYER2_CD and LOBBYIST_EMPLOYER3_CD
    tables.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=96, end_page=97),
    ]
    contributor_id = fields.IntegerField(
        db_column="CONTRIBUTOR_ID",
        help_text="Contributor identification number."
    )
    current_qtr_amt = fields.IntegerField(
        db_column="CURRENT_QTR_AMT",
        help_text="Current Quarter Amount"
    )
    employer_id = fields.IntegerField(
        db_column="EMPLOYER_ID",
        help_text="Employer identification number."
    )
    employer_name = fields.CharField(
        db_column="EMPLOYER_NAME",
        max_length=300,
        blank=True,
        help_text="Employer Name"
    )
    INTEREST_CD_CHOICES = (
        (40301, 'AGRICULTURE'),
        (40302, 'EDUCATION'),
        (40303, 'ENTERTAINMENT/RECREATION'),
        (40304, 'FINANCE/INSURANCE'),
        (40305, 'GOVERNMENT'),
        (40306, 'HEALTH'),
        (40307, 'LABOR UNIONS'),
        (40308, 'LEGAL'),
        (40309, 'LODGING/RESTAURANTS'),
        (40310, 'MANUFACTURING/INDUSTRIAL'),
        (40311, 'MERCHANDISE/RETAIL'),
        (40312, 'MISCELLANEOUS'),
        (40313, 'OIL AND GAS'),
        (40314, 'POLITICAL ORGANIZATIONS'),
        (40315, 'PROFESSIONAL/TRADE'),
        (40316, 'PUBLIC EMPLOYEES'),
        (40317, 'REAL ESTATE'),
        (40318, 'TRANSPORTATION'),
        (40319, 'UTILITIES'),
    )
    interest_cd = fields.IntegerField(
        db_column="INTEREST_CD",
        choices=INTEREST_CD_CHOICES,
        verbose_name='interest code',
        help_text='Interest Code',
        documentcloud_pages=[
            DocumentCloud(id='2774529-Lookup-Codes-Cd', start_page=19),
        ],
    )
    interest_name = fields.CharField(
        db_column="INTEREST_NAME",
        max_length=300,
        blank=True,
        verbose_name="Interest name.",
        help_text="Interest name.",
    )
    qtr_1 = fields.IntegerField(
        db_column="QTR_1",
        verbose_name="quarter 1 amount",
        help_text="Quarter 1 total amount.",
    )
    qtr_2 = fields.IntegerField(
        db_column="QTR_2",
        verbose_name="quarter 2 amount.",
        help_text="Quarter 2 total amount.",
    )
    qtr_3 = fields.IntegerField(
        db_column="QTR_3",
        verbose_name="quarter 3 amount",
        help_text="Quarter 3 total amount.",
    )
    qtr_4 = fields.IntegerField(
        db_column="QTR_4",
        verbose_name="quarter 4 amount",
        help_text="Quarter 4 total amount.",
    )
    qtr_5 = fields.IntegerField(
        db_column="QTR_5",
        verbose_name="quarter 5 amount5",
        help_text="Quarter 5 total amount.",
    )
    qtr_6 = fields.IntegerField(
        db_column="QTR_6",
        verbose_name="quarter 6 amount.",
        help_text="Quarter 6 total amount.",
    )
    qtr_7 = fields.IntegerField(
        db_column="QTR_7",
        verbose_name="quarter 7 amount.",
        help_text="Quarter 7 total amount.",
    )
    qtr_8 = fields.IntegerField(
        db_column="QTR_8",
        verbose_name="quarter 8 amount.",
        help_text="Quarter 8 total amount.",
    )
    session_id = fields.IntegerField(
        db_column="SESSION_ID",
        verbose_name="session identification number.",
        help_text="Session identification number.",
    )
    session_total_amt = fields.IntegerField(
        db_column="SESSION_TOTAL_AMT",
        verbose_name="session total amount",
        help_text="Total amount for the session.",
    )
    session_yr_1 = fields.IntegerField(
        db_column="SESSION_YR_1",
        verbose_name="session year 1",
        help_text="Total amount for year 1 of the session.",
    )
    session_yr_2 = fields.IntegerField(
        db_column="SESSION_YR_2",
        verbose_name="session year 2",
        help_text="Total amount for year 2 of the session.",
    )
    yr_1_ytd_amt = fields.IntegerField(
        db_column="YR_1_YTD_AMT",
        help_text="Year 1 year to date amount.",
    )
    yr_2_ytd_amt = fields.IntegerField(
        db_column="YR_2_YTD_AMT",
        help_text="Year 2 year to date amount.",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_EMPLOYER_HISTORY_CD'

    def __str__(self):
        return str(self.contributor_id)


@python_2_unicode_compatible
class LobbyistFirm1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM2_CD and LOBBYIST_FIRM3_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the distinct SESSION_YR_1 values are 2001, and the distinct SESSION_YR_2
    are 2002.
    """
    UNIQUE_KEY = False
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=103, end_page=104),
    ]
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM1_CD'

    def __str__(self):
        return str(self.firm_id)


@python_2_unicode_compatible
class LobbyistFirm2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM1_CD and LOBBYIST_FIRM3_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the distinct SESSION_YR_1 values are 2001, and the distinct SESSION_YR_2
    are 2002.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=104),
    ]
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM2_CD'

    def __str__(self):
        return str(self.firm_id)


@python_2_unicode_compatible
class LobbyistFirm3Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM1_CD and LOBBYIST_FIRM2_CD.

    All three tables are documented in "Cal-Access Tables, Columns, Indexes", but
    with this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, the distinct SESSION_YR_1 values are 2001, and the distinct SESSION_YR_2
    are 2002.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=105),
    ]
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    current_qtr_amt = fields.FloatField(
        db_column='CURRENT_QTR_AMT',
        help_text="Current quarter amount"
    )
    session_total_amt = fields.FloatField(
        db_column='SESSION_TOTAL_AMT',
        help_text="Total amount for the session"
    )
    contributor_id = fields.IntegerField(
        db_column='CONTRIBUTOR_ID',
        blank=True,
        null=True,
        verbose_name="contributor ID",
        help_text="Contributor identification number"
    )
    session_yr_1 = fields.IntegerField(
        db_column='SESSION_YR_1',
        help_text="Total amount of year 1 of the session",
    )
    session_yr_2 = fields.IntegerField(
        db_column='SESSION_YR_2',
        help_text="Total amount of year 2 of the session",
    )
    yr_1_ytd_amt = fields.FloatField(
        db_column='YR_1_YTD_AMT',
        verbose_name="Year 1 year-to-date-amount",
        help_text="Year 1 year-to-date-amount",
    )
    yr_2_ytd_amt = fields.FloatField(
        db_column='YR_2_YTD_AMT',
        verbose_name="Year 2 year-to-date-amount",
        help_text="Year 2 year-to-date-amount",
    )
    qtr_1 = fields.FloatField(
        db_column='QTR_1',
        verbose_name="Quarter 1",
        help_text="Quarter total amount",
    )
    qtr_2 = fields.FloatField(
        db_column='QTR_2',
        verbose_name="Quarter 2",
        help_text="Quarter total amount",
    )
    qtr_3 = fields.FloatField(
        db_column='QTR_3',
        verbose_name="Quarter 3",
        help_text="Quarter total amount",
    )
    qtr_4 = fields.FloatField(
        db_column='QTR_4',
        verbose_name="Quarter 4",
        help_text="Quarter total amount",
    )
    qtr_5 = fields.FloatField(
        db_column='QTR_5',
        verbose_name="Quarter 5",
        help_text="Quarter total amount",
    )
    qtr_6 = fields.FloatField(
        db_column='QTR_6',
        verbose_name="Quarter 6",
        help_text="Quarter total amount",
    )
    qtr_7 = fields.FloatField(
        db_column='QTR_7',
        verbose_name="Quarter 7",
        help_text="Quarter total amount",
    )
    qtr_8 = fields.FloatField(
        db_column='QTR_8',
        verbose_name="Quarter 8",
        help_text="Quarter total amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM3_CD'

    def __str__(self):
        return str(self.firm_id)


@python_2_unicode_compatible
class LobbyistFirmEmployer1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM_EMPLOYER2_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also RPT_START and RPT_END each contain only one distinct value, "2001-04-01"
    and "2001-06-30", respectively.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=100),
    ]
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    filing_id = fields.IntegerField(
        db_column='FILING_ID',
        db_index=True,
        verbose_name='filing ID',
        help_text="Unique filing identification number"
    )
    filing_sequence = fields.IntegerField(
        db_column='FILING_SEQUENCE',
        help_text="Amendment number. 0 is the original filing. \
1 to 999 are amendments"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    rpt_start = fields.DateField(
        db_column='RPT_START',
        null=True,
        help_text="Starting date for the period the report covers"
    )
    rpt_end = fields.DateField(
        db_column='RPT_END',
        null=True,
        help_text="Ending date for the period the report covers"
    )
    per_total = fields.FloatField(
        db_column='PER_TOTAL',
        help_text="Total this reporting period"
    )
    cum_total = fields.FloatField(
        db_column='CUM_TOTAL',
        help_text='Cumulative total to date'
    )
    lby_actvty = fields.CharField(
        db_column='LBY_ACTVTY',
        max_length=182,
        blank=True,
        help_text="Description of lobbying activity"
    )
    ext_lby_actvty = fields.CharField(
        db_column='EXT_LBY_ACTVTY',
        max_length=32,
        blank=True,
        help_text="This field is undocumented"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM_EMPLOYER1_CD'
        ordering = ("-rpt_start",)

    def __str__(self):
        return str(self.firm_id)


@python_2_unicode_compatible
class LobbyistFirmEmployer2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM_EMPLOYER1_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also RPT_START and RPT_END each contain only one distinct value, "2001-04-01"
    and "2001-06-30", respectively.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=11, end_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=100, end_page=101),
    ]
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    filing_id = fields.IntegerField(
        db_column='FILING_ID',
        db_index=True,
        verbose_name='filing ID',
        help_text="Unique filing identification number"
    )
    filing_sequence = fields.IntegerField(
        db_column='FILING_SEQUENCE',
        help_text="Amendment number. 0 is the original filing. \
1 to 999 are amendments"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    employer_name = fields.CharField(
        db_column='EMPLOYER_NAME',
        max_length=300,
        help_text="Employer name"
    )
    rpt_start = fields.DateField(
        db_column='RPT_START',
        null=True,
        help_text="Starting date for the period the report covers"
    )
    rpt_end = fields.DateField(
        db_column='RPT_END',
        null=True,
        help_text="Ending date for the period the report covers"
    )
    per_total = fields.FloatField(
        db_column='PER_TOTAL',
        help_text="Total this reporting period"
    )
    cum_total = fields.FloatField(
        db_column='CUM_TOTAL',
        help_text='Cumulative total to date'
    )
    lby_actvty = fields.CharField(
        db_column='LBY_ACTVTY',
        max_length=182,
        blank=True,
        help_text="Description of lobbying activity"
    )
    ext_lby_actvty = fields.CharField(
        db_column='EXT_LBY_ACTVTY',
        max_length=32,
        blank=True,
        help_text="This field is undocumented"
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM_EMPLOYER2_CD'
        ordering = ("-rpt_start",)

    def __str__(self):
        return str(self.filing_id)


@python_2_unicode_compatible
class LobbyistFirmHistoryCd(CalAccessBaseModel):
    """
    Undocumented.

    An empty file of the same name is included in the Secretary of State's daily CAL-ACCESS database exports.

    This table is documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation
    on these tables. Cox 5/11/2000".

    Also, the columns on this table are identical to the columns on the
    LOBBYIST_FIRM1_CD, LOBBYIST_FIRM2_CD and LOBBYIST_FIRM3_CD tables.
    """
    UNIQUE_KEY = (
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=101, end_page=102),
    ]
    contributor_id = fields.IntegerField(
        db_column="CONTRIBUTOR_ID",
        help_text="Contributor identification number."
    )
    current_qtr_amt = fields.IntegerField(
        db_column="CURRENT_QTR_AMT",
        help_text="Current Quarter Amount"
    )
    firm_id = fields.IntegerField(
        db_column="FIRM_ID",
        help_text="Identification number of the Firm/Employer/Coalition."
    )
    firm_name = fields.CharField(
        db_column="FIRM_NAME",
        max_length=300,
        help_text="Name of Firm/Employer/Coalition"
    )
    qtr_1 = fields.IntegerField(
        db_column="QTR_1",
        help_text="Quarter total amount."
    )
    qtr_2 = fields.IntegerField(
        db_column="QTR_2",
        help_text="Quarter total amount."
    )
    qtr_3 = fields.IntegerField(
        db_column="QTR_3",
        help_text="Quarter total amount."
    )
    qtr_4 = fields.IntegerField(
        db_column="QTR_4",
        help_text="Quarter total amount."
    )
    qtr_5 = fields.IntegerField(
        db_column="QTR_5",
        help_text="Quarter total amount."
    )
    qtr_6 = fields.IntegerField(
        db_column="QTR_6",
        help_text="Quarter total amount."
    )
    qtr_7 = fields.IntegerField(
        db_column="QTR_7",
        help_text="Quarter total amount."
    )
    qtr_8 = fields.IntegerField(
        db_column="QTR_8",
        help_text="Quarter total amount."
    )
    session_id = fields.IntegerField(
        db_column="SESSION_ID",
        help_text="Session identification number."
    )
    session_total_amt = fields.IntegerField(
        db_column="SESSION_TOTAL_AMT",
        help_text="Total amount for the session."
    )
    session_yr_1 = fields.IntegerField(
        db_column="SESSION_YR_1",
        help_text="Total amount for year 1 of the session."
    )
    session_yr_2 = fields.IntegerField(
        db_column="SESSION_YR_2",
        help_text="Total amount for year 2 of the session."
    )
    yr_1_ytd_amt = fields.IntegerField(
        db_column="YR_1_YTD_AMT",
        verbose_name="Year 1 year to date amount.",
        help_text="Year 1 year to date amount.",
    )
    yr_2_ytd_amt = fields.IntegerField(
        db_column="YR_2_YTD_AMT",
        verbose_name="Year 2 year to date amount",
        help_text="Year 2 year to date amount",
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM_HISTORY_CD'

    def __str__(self):
        return str(self.contributor_id)


@python_2_unicode_compatible
class LobbyistFirmLobbyist1Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM_LOBBYIST2_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, all rows have the same SESSION_ID value: 2001.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=102),
    ]
    lobbyist_id = fields.IntegerField(
        db_column='LOBBYIST_ID',
        verbose_name="Lobbyist ID",
        help_text="Lobbyist identification number"
    )
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    lobbyist_last_name = fields.CharField(
        db_column='LOBBYIST_LAST_NAME',
        max_length=15,
        help_text="Lobbyist last name"
    )
    lobbyist_first_name = fields.CharField(
        db_column='LOBBYIST_FIRST_NAME',
        max_length=17,
        help_text="Lobbyist first name"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM_LOBBYIST1_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.lobbyist_id)


@python_2_unicode_compatible
class LobbyistFirmLobbyist2Cd(CalAccessBaseModel):
    """
    Deprecated table for the disclosure of lobbyist relationships.

    This table is identical to LOBBYIST_FIRM_LOBBYIST1_CD.

    Both tables are documented in "Cal-Access Tables, Columns, Indexes", but with
    this cryptic note: "Matt needs to describe the relationship between the
    multiple tables. Documentation should be cloned from D H's documentation on
    these tables. Cox 5/11/2000".

    Also, all rows have the same SESSION_ID value: 2001.
    """
    UNIQUE_KEY = False
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=12),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=102, end_page=103),
    ]
    lobbyist_id = fields.IntegerField(
        db_column='LOBBYIST_ID',
        verbose_name="Lobbyist ID",
        help_text="Lobbyist identification number"
    )
    firm_id = fields.IntegerField(
        db_column='FIRM_ID',
        verbose_name="Firm ID",
        help_text="Identification number of the firm, employer or coalition"
    )
    lobbyist_last_name = fields.CharField(
        db_column='LOBBYIST_LAST_NAME',
        max_length=15,
        help_text="Lobbyist last name"
    )
    lobbyist_first_name = fields.CharField(
        db_column='LOBBYIST_FIRST_NAME',
        max_length=17,
        help_text="Lobbyist first name"
    )
    firm_name = fields.CharField(
        db_column='FIRM_NAME',
        max_length=400,
        help_text="Name of firm, employer or coalition",
    )
    session_id = fields.IntegerField(
        verbose_name='session ID',
        db_column='SESSION_ID',
        help_text='Legislative session identification number',
        null=True,
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'LOBBYIST_FIRM_LOBBYIST2_CD'
        ordering = ("-session_id",)

    def __str__(self):
        return str(self.lobbyist_id)


@python_2_unicode_compatible
class EfsFilingLogCd(CalAccessBaseModel):
    """
    Logs from the Electronic Filing Subsystem, which accepts and validates electronic filings.
    """
    UNIQUE_KEY = (
        "FILING_DATE",
        "VENDOR"
    )
    DOCUMENTCLOUD_PAGES = [
        DocumentCloud(id='2711624-Overview', start_page=1, end_page=2),
        DocumentCloud(id='2711614-CalAccessTablesWeb', start_page=49, end_page=50),
    ]
    FILING_FORMS = [
        annotations.get_form('F400'),
        annotations.get_form('F401'),
        annotations.get_form('F402'),
        annotations.get_form('F410'),
        annotations.get_form('F425'),
        annotations.get_form('F450'),
        annotations.get_form('F460'),
        annotations.get_form('F461'),
        annotations.get_form('F465'),
        annotations.get_form('F496'),
        annotations.get_form('F497'),
        annotations.get_form('F498'),
        annotations.get_form('F601'),
        annotations.get_form('F602'),
        annotations.get_form('F603'),
        annotations.get_form('F604'),
        annotations.get_form('F606'),
        annotations.get_form('F607'),
        annotations.get_form('F615'),
        annotations.get_form('F625'),
        annotations.get_form('F635'),
        annotations.get_form('F645'),
    ]
    filing_date = fields.DateField(
        db_column='FILING_DATE',
        null=True,
        help_text="Date of filing"
    )
    filingstatus = fields.IntegerField(
        db_column='FILINGSTATUS',
        help_text="Status of filing. This field is described in the docs as being\
VARCHAR. However, its distinct values are 0, 1, 2 and 7.",
    )
    vendor = fields.CharField(
        db_column='VENDOR',
        max_length=250,
        help_text="Software vendor who submitted the electronic filing"
    )
    filer_id = fields.CharField(
        verbose_name='filer ID',
        db_column='FILER_ID',
        max_length=250,
        blank=True,
        db_index=True,
        help_text="Filer's unique identification number",
    )
    FORM_TYPE_CHOICES = tuple([(f.db_value, f.full_title) for f in FILING_FORMS]) + (
        ('BADFORMAT 253', 'Unknown'),
        ('form', 'Unknown'),
    )
    form_type = fields.CharField(
        db_column='FORM_TYPE',
        max_length=250,
        help_text='Name of the source filing form or schedule',
        db_index=True,
        choices=FORM_TYPE_CHOICES,
        verbose_name="form type",
        documentcloud_pages=[
            DocumentCloud(id='2711624-Overview', start_page=4, end_page=8),
        ]
    )
    error_no = fields.CharField(
        db_column='ERROR_NO',
        max_length=250,
        help_text='Most records have a value of "ACCEPTED". Other records include "ERROR"\
or "BADFORMAT" and a three-digit number.',
    )

    class Meta(CalAccessBaseModel.Meta):
        """
        Meta model options.
        """
        db_table = 'EFS_FILING_LOG_CD'
        ordering = ("-filing_date",)

    def __str__(self):
        return "{} ({})".format(self.vendor, self.filing_date)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-08-02 21:01
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0001_squashed_0014_auto_20160801_2039'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rawdataversion',
            options={'get_latest_by': 'release_datetime', 'ordering': ('-release_datetime',), 'verbose_name': 'CAL-ACCESS raw data version'},
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Base models for tables from the CAL-ACCESS database.
"""
from __future__ import unicode_literals

# Text
import textwrap

# Models
from django.db import models
from calaccess_raw import managers
from django.db.models.base import ModelBase


class CalAccessMetaClass(ModelBase):
    """
    A custom metaclass for our base model.

    Automatically configures Meta attributes common to all models.
    """
    def __new__(cls, name, bases, attrs):
        """
        Override the default __new__ behavior.
        """
        klass = super(CalAccessMetaClass, cls).__new__(cls, name, bases, attrs)

        # Cook up an automated verbose name for each model
        klass_group = str(klass).split(".")[-2].upper()
        klass_table = klass._meta.db_table
        klass_name = "{0}: {1}".format(klass_group, klass_table)

        # Insert the verbose name into each model's configuration
        klass._meta.verbose_name = klass_name
        klass._meta.verbose_name_plural = klass_name

        # Set the app_label too
        klass._meta.app_label = "calaccess_raw"

        # Finish up
        return klass


class CalAccessBaseModel(models.Model):
    """
    An abstract model with some tricks we'll reuse.
    """
    __metaclass__ = CalAccessMetaClass

    # The UNIQUE_KEY is one or more fields that, taken together, are unique
    # within the database. https://en.wikipedia.org/wiki/Unique_key

    # Because the CAL-ACCESS database is released without unique keys specified
    # we determine these on our own and list them with each model.

    # If a single field is believed to be unique, it can be set as a simple
    # string variable, like:

    #   UNIQUE_KEY = 'field_name'

    # If multiple fields must be combined to guarantee uniqueness, they
    # should be listed as tuple like:

    #   UNIQUE_KEY = ('field_one', 'field_two')

    # If the unique key does not exist or cannot be determined it will be
    # set to False

    #   UNIQUE_KEY = False
    UNIQUE_KEY = None

    # A list of URL strings that point to pages hosted on DocumentCloud.org
    # that contain documentation for this model. Once assembled they can be
    # embedded in our user-facing documentation as images.

    # Should be filled with instances of our DocumentCloud class below
    # which accepts a unique DocumentCloud id along with start and/or end
    # page numbers
    DOCUMENTCLOUD_PAGES = []
    FILING_FORMS = []

    # Default manager
    objects = managers.CalAccessManager()

    def doc(self):
        """
        Return the model's docstring as a readable string ready to print.
        """
        if self.__doc__.startswith(self.klass_name):
            return ''
        return textwrap.dedent(self.__doc__).strip()

    @property
    def db_table(self):
        """
        Return the model's database table name as a string.
        """
        return self._meta.db_table

    @property
    def klass(self):
        """
        Return the model class itself.
        """
        return self.__class__

    @property
    def klass_name(self):
        """
        Return the name of the model class.
        """
        return self.__class__.__name__

    @property
    def klass_group(self):
        """
        Return the name of the model's group, as determined by its submodule.
        """
        return str(self.__class__).split(".")[-2]

    def get_field_list(self):
        """
        Return all the fields on the model as a list.
        """
        return self._meta.fields

    def get_csv_name(self):
        """
        Return the name of the clean CSV file that contains the model's data.
        """
        return self.__class__.objects.get_csv_name()

    def get_csv_path(self):
        """
        Return the path to the clean CSV file that contains the model's data.
        """
        return self.__class__.objects.get_csv_path()

    def get_tsv_name(self):
        """
        Return the name of the raw TSV file that contains the model's data.
        """
        return self.__class__.objects.get_tsv_name()

    def get_tsv_path(self):
        """
        Return the path to the raw TSV file that contains the model's data.
        """
        return self.__class__.objects.get_tsv_path()

    def get_unique_key_list(self):
        """
        Return UNIQUE_KEY setting as a list regardless of its data type.
        """
        if self.__class__.UNIQUE_KEY is None:
            return []
        elif self.__class__.UNIQUE_KEY is False:
            return []
        elif isinstance(self.__class__.UNIQUE_KEY, (list, tuple)):
            return self.__class__.UNIQUE_KEY
        else:
            return [self.__class__.UNIQUE_KEY]

    def get_documentcloud_pages(self):
        """
        Return a list of tuples for each page or each document in the DOCUMENTCLOUD_PAGES attr.

        Each tuple contains a DocumentCloud and DocPage object.
        """
        page_list = []
        for dc in self.DOCUMENTCLOUD_PAGES:
            page_list.extend([(dc, page) for page in dc.pages])
        return page_list

    def get_filing_forms_w_sections(self):
        """
        Returns a list of tuples, each containing a FilingForm object and list of FilingFormSection objects.
        """
        from calaccess_raw.annotations import FilingForm
        forms_dict = {}
        for i in self.FILING_FORMS:
            if isinstance(i, FilingForm):
                try:
                    forms_dict[i]
                except KeyError:
                    forms_dict[i] = []
            else:
                try:
                    forms_dict[i.form].append(i)
                except KeyError:
                    forms_dict[i.form] = [i]
        return sorted(forms_dict.items(), key=lambda x: x[0].id)

    class Meta:
        """
        Meta model options.
        """
        abstract = True
        app_label = 'calaccess_raw'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-11-23 22:19
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0012_auto_20161123_2217'),
    ]

    operations = [
        migrations.AlterField(
            model_name='expncd',
            name='expn_code',
            field=calaccess_raw.fields.CharField(blank=True, choices=[(b'CMP', b'campaign paraphernalia/miscellaneous'), (b'CNS', b'campaign consultants'), (b'CTB', b'contribution (if nonmonetary, explain)*'), (b'CVC', b'civic donations'), (b'FIL', b'candidate filing/ballot feeds'), (b'FND', b'fundraising events'), (b'IKD', b'In-kind contribution (nonmonetary)'), (b'IND', b'independent expenditure supporting/opposing others (explain)*'), (b'LEG', b'legal defense'), (b'LIT', b'campaign literature and mailings'), (b'LON', b'loan'), (b'MBR', b'member communications'), (b'MON', b'monetary contribution'), (b'MTG', b'meetings and appearances'), (b'OFC', b'office expenses'), (b'PET', b'petition circulating'), (b'PHO', b'phone banks'), (b'POL', b'polling and survey research'), (b'POS', b'postage, delivery and messenger services'), (b'PRO', b'professional services (legal, accounting)'), (b'PRT', b'print ads'), (b'RAD', b'radio airtime and production costs'), (b'RFD', b'returned contributions'), (b'SAL', b'campaign workers salaries'), (b'TEL', b'T.V. or cable airtime and production costs'), (b'TRC', b'candidate travel, lodging and meals (explain)'), (b'TRS', b'staff/spouse travel, lodging and meals (explain)'), (b'TSF', b'transfer between committees of the same candidate/sponsor'), (b'VOT', b'voter registration'), (b'WEB', b'information technology costs (internet, e-mail)'), ('ctb', b'contribution (if nonmonetary, explain)*'), ('ikd', b'In-kind contribution (nonmonetary)'), ('Mon', b'monetary contribution'), ('ofc', b'office expenses'), ('OFc', b'office expenses'), ('Ofc', b'office expenses'), ('', 'Unknown'), ('*', 'Unknown'), ('0', 'Unknown'), ('001', 'Unknown'), ('011', 'Unknown'), ('200', 'Unknown'), ('401', 'Unknown'), ('ADV', 'Unknown'), ('ANN', 'Unknown'), ('APR', 'Unknown'), ('AUG', 'Unknown'), ('AUT', 'Unknown'), ('Ban', 'Unknown'), ('BAN', 'Unknown'), ('BOO', 'Unknown'), ('BOX', 'Unknown'), ('C', 'Unknown'), ('CAT', 'Unknown'), ('CC', 'Unknown'), ('CHE', 'Unknown'), ('CIV', 'Unknown'), ('CNT', 'Unknown'), ('CON', 'Unknown'), ('COP', 'Unknown'), ('CRE', 'Unknown'), ('CSN', 'Unknown'), ('CT', 'Unknown'), (',CT', 'Unknown'), ('.CT', 'Unknown'), ('CTN', 'Unknown'), ('CVD', 'Unknown'), ('DAT', 'Unknown'), ('DEC', 'Unknown'), ('Dem', 'Unknown'), ('DIN', 'Unknown'), ('Don', 'Unknown'), ('DON', 'Unknown'), ('Ear', 'Unknown'), ('EIM', 'Unknown'), ('EMP', 'Unknown'), ('F', 'Unknown'), ('FAX', 'Unknown'), ('FDN', 'Unknown'), ('FED', 'Unknown'), ('FEE', 'Unknown'), ('FIN', 'Unknown'), ('Fun', 'Unknown'), ('FUN', 'Unknown'), ('G', 'Unknown'), ('GEN', 'Unknown'), ('GGG', 'Unknown'), ('GOT', 'Unknown'), ('IEs', 'Unknown'), ('IN-', 'Unknown'), ('Ina', 'Unknown'), ('INK', 'Unknown'), ('INS', 'Unknown'), ('ITE', 'Unknown'), ('JAN', 'Unknown'), ('JUL', 'Unknown'), ('JUN', 'Unknown'), ('KIC', 'Unknown'), ('L', 'Unknown'), ('LEV', 'Unknown'), ('Lit', 'Unknown'), ('LN#', 'Unknown'), ('LOG', 'Unknown'), ('M', 'Unknown'), ('MAI', 'Unknown'), ('Mar', 'Unknown'), ('MAR', 'Unknown'), ('MAY', 'Unknown'), ('MED', 'Unknown'), ('MEE', 'Unknown'), ('MGT', 'Unknown'), ('Mis', 'Unknown'), ('MRB', 'Unknown'), ('NGP', 'Unknown'), ('NON', 'Unknown'), ('NOT', 'Unknown'), ('NOV', 'Unknown'), ('O', 'Unknown'), ('OCT', 'Unknown'), ('.OF', 'Unknown'), ('OFF', 'Unknown'), ('OPE', 'Unknown'), ('OTH', 'Unknown'), ('P', 'Unknown'), ('Pac', 'Unknown'), ('PAI', 'Unknown'), ('PAR', 'Unknown'), ('PAY', 'Unknown'), ('PEN', 'Unknown'), ('PMT', 'Unknown'), ('.PO', 'Unknown'), ('Pos', 'Unknown'), ('PRE', 'Unknown'), ('PRI', 'Unknown'), ('PRP', 'Unknown'), ('R', 'Unknown'), ('.Re', 'Unknown'), ('.RE', 'Unknown'), ('REF', 'Unknown'), ('REI', 'Unknown'), ('RFP', 'Unknown'), ('S', 'Unknown'), ('S-A', 'Unknown'), ('SA', 'Unknown'), ('Sal', 'Unknown'), ('S C', 'Unknown'), ('S.C', 'Unknown'), ('SCU', 'Unknown'), ('SEE', 'Unknown'), ('SEN', 'Unknown'), ('SEP', 'Unknown'), ('S.M.', 'Unknown'), ('SOF', 'Unknown'), ('SWI', 'Unknown'), ('T', 'Unknown'), ('TAX', 'Unknown'), ('TB', 'Unknown'), ('TB,', 'Unknown'), ('TIC', 'Unknown'), ('Tor', 'Unknown'), ('TRA', 'Unknown'), ('TRF', 'Unknown'), ('TRV', 'Unknown'), ('UN', 'Unknown'), ('UTI', 'Unknown'), ('V', 'Unknown'), ('VEN', 'Unknown'), ('-VO', 'Unknown'), ('VOI', 'Unknown'), ('VOY', 'Unknown'), ('WI', 'Unknown'), ('x', 'Unknown'), ('X', 'Unknown'), ('S-6', 'Unknown'), ('S.M', 'Unknown'), ('S-4', 'Unknown'), ('SA:', 'Unknown'), ('100', 'Unknown'), ('RFN', 'Unknown'), ('REN', 'Unknown'), ('003', 'Unknown'), ('S-1', 'Unknown'), ('08', 'Unknown')], db_column='EXPN_CODE', documentcloud_pages=[calaccess_raw.annotations.DocumentCloud(id=b'2712033-Cal-Format-1-05-02', start_page=11), calaccess_raw.annotations.DocumentCloud(end_page=14, id=b'2712034-Cal-Format-201', start_page=13)], help_text='The type of expenditure', max_length=3, verbose_name='expense code'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utilities for representing CAL-ACCESS forms.
"""
from __future__ import unicode_literals
from .documents import DocumentCloud
from calaccess_raw import get_model_list
from django.utils.deconstruct import deconstructible


@deconstructible
class FilingForm(object):
    """
    A form used to collect information for the CAL-ACCESS database.
    """
    def __init__(self, id, title, **kwargs):
        """
        Create a new object by submitting a unique ID and title.
        """
        self.id = id
        self.title = title
        self.description = kwargs.get('description')
        self.group = kwargs.get('group')
        self.documentcloud_id = kwargs.get('documentcloud_id')
        self.db_value = kwargs.get('db_value', self.id)
        self.sections = []
        if self.documentcloud_id:
            self.documentcloud = DocumentCloud(self.documentcloud_id)
        else:
            self.documentcloud = None

    @property
    def type_and_num(self):
        """
        Returns a short title for the form that includes its type and number.
        """
        if self.id[0] == 'E':
            self._type_and_num = 'Electronic Form {}'.format(self.id[1:])
        elif self.id[0] == 'S':
            self._type_and_num = 'Schedule {}'.format(self.id[1:])
        else:
            self._type_and_num = 'Form {}'.format(self.id[1:])

        return self._type_and_num

    @property
    def full_title(self):
        """
        Returns the full title of the form.
        """
        self._full_title = '{0}: {1}'.format(self.type_and_num, self.title)
        return self._full_title

    def add_section(self, id, title, **kwargs):
        """
        Adds a Section with the provided title and options to this object.
        """
        new_section = FilingFormSection(
            form=self,
            id=id,
            title=title,
            db_value=kwargs.get('db_value', self.db_value),
            start_page=kwargs.get('start_page'),
            end_page=kwargs.get('end_page'),
            documentcloud_id=kwargs.get('documentcloud_id'),
        )
        self.sections.append(new_section)
        return new_section

    def get_section(self, id):
        """
        Returns the Section object with the given id.
        """
        section_dict = {i.id: i for i in self.sections}
        return section_dict[id]

    def get_models(self):
        """
        Returns all the CAL-ACCESS models connected with this form.
        """
        models = []
        for model in get_model_list():
            if self in [x[0] for x in model().get_filing_forms_w_sections()]:
                models.append(model)

        return models

    def __str__(self):
        return str(self.id)


@deconstructible
class FilingFormSection(object):
    """
    A section of a FilingForm (e.g., a cover page, summary sheet, schedule or part).
    """
    def __init__(self, form, id, title, **kwargs):
        """
        Create a new object by submitting a FilingForm parent with an ID and title.
        """
        self.form = form
        self.id = id
        self.title = title
        self.db_value = kwargs.get('db_value', form.db_value)
        self.start_page = kwargs.get('start_page')
        self.end_page = kwargs.get('end_page')
        self.documentcloud = DocumentCloud(
            self.form.documentcloud_id,
            self.start_page,
            self.end_page
        )

    @property
    def full_title(self):
        """
        Returns full title of the section, including the parent form's name.
        """
        self._full_title = '{0} ({1}): {2}'.format(
            self.form.type_and_num,
            self.form.title,
            self.title,
        )
        return self._full_title

    def __str__(self):
        return str(self.id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom administration panels for campaign models.
"""
from __future__ import unicode_literals
from django.contrib import admin
from calaccess_raw import models
from .base import BaseAdmin


@admin.register(models.CvrSoCd)
class CvrSoCdAdmin(BaseAdmin):
    """
    Custom admin for the CvrSoCd model.
    """
    list_display = ("filing_id", "amend_id", "rpt_date", "filer_naml", "form_type")
    date_hierarchy = "rpt_date"


@admin.register(models.Cvr2SoCd)
class Cvr2SoCdAdmin(BaseAdmin):
    """
    Custom admin for the Cvr2SoCd model.
    """
    list_display = ("filing_id", "item_cd", "entity_cd", "enty_naml", "form_type")


@admin.register(models.CvrCampaignDisclosureCd)
class CvrCampaignDisclosureCdAdmin(BaseAdmin):
    """
    Custom admin for the CvrCampaignDisclosureCd model.
    """
    list_display = ("filing_id", "rpt_date", "filer_naml", "cmtte_type", "form_type")
    date_hierarchy = "rpt_date"


@admin.register(models.Cvr2CampaignDisclosureCd)
class Cvr2CampaignDisclosureCdAdmin(BaseAdmin):
    """
    Custom admin for the Cvr2CampaignDisclosureCd model.
    """
    list_display = ("filing_id", "enty_naml", "form_type")


@admin.register(models.Cvr3VerificationInfoCd)
class Cvr3VerificationInfoCdAdmin(BaseAdmin):
    """
    Custom admin for the Cvr3VerificationInfoCd model.
    """
    list_display = ("filing_id", "sig_date", "sig_naml", "form_type")
    date_hierarchy = "sig_date"


@admin.register(models.DebtCd)
class DebtCdAdmin(BaseAdmin):
    """
    Custom admin for the DebtCd model.
    """
    list_display = (
        "filing_id",
        "expn_code",
        "payee_naml",
        "amt_incur",
        "amt_paid",
    )


@admin.register(models.ExpnCd)
class ExpnCdAdmin(BaseAdmin):
    """
    Custom admin for the ExpnCd model.
    """
    list_display = (
        "filing_id",
        "expn_date",
        "cand_naml",
        "payee_naml",
        "form_type",
        "amount",
    )
    date_hierarchy = "expn_date"


@admin.register(models.LoanCd)
class LoanCdAdmin(BaseAdmin):
    """
    Custom admin for the LoanCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "loan_date1",
        "loan_type",
        "lndr_naml",
        "loan_amt1",
        "loan_amt2",
        "loan_amt3",
        "loan_amt4"
    )
    date_hierarchy = "loan_date1"


@admin.register(models.RcptCd)
class RcptCdAdmin(BaseAdmin):
    """
    Custom admin for the RcptCd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "rcpt_date",
        "ctrib_naml",
        "ctrib_emp",
        "ctrib_occ",
        "amount"
    )
    date_hierarchy = "rcpt_date"


@admin.register(models.S401Cd)
class S401CdAdmin(BaseAdmin):
    """
    Custom admin for the S401Cd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "cand_naml",
        "payee_naml",
        "amount",
    )


@admin.register(models.F495P2Cd)
class F495P2CdAdmin(BaseAdmin):
    """
    Custom admin for the F495P2Cd model.
    """
    list_display = (
        "filing_id",
        "form_type",
        "elect_date",
        "contribamt"
    )
    date_hierarchy = "elect_date"


@admin.register(models.S496Cd)
class S496CdAdmin(BaseAdmin):
    """
    Custom admin for the S496Cd model.
    """
    list_display = (
        "filing_id",
        "exp_date",
        "expn_dscr",
        "amount"
    )
    date_hierarchy = "exp_date"


@admin.register(models.S497Cd)
class S497CdAdmin(BaseAdmin):
    """
    Custom admin for the S497Cd model.
    """
    list_display = (
        "filing_id",
        "ctrib_date",
        "cand_naml",
        "enty_naml",
        "amount",
    )
    date_hierarchy = "ctrib_date"


@admin.register(models.S498Cd)
class S498CdAdmin(BaseAdmin):
    """
    Custom admin for the S498Cd model.
    """
    list_display = (
        "filing_id",
        "date_rcvd",
        "payor_naml",
        "cand_naml",
        "amt_rcvd",
    )
    date_hierarchy = "date_rcvd"


@admin.register(models.F501502Cd)
class F501502CdAdmin(BaseAdmin):
    """
    Custom admin for the F501502Cd model.
    """
    list_display = (
        "filing_id",
        "rpt_date",
        "cand_naml",
        "office_cd",
        "elec_type",
    )
    date_hierarchy = "rpt_date"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Extract the CAL-ACCESS raw data files from downloaded ZIP.
"""
from __future__ import unicode_literals

# Files
import os
import shutil
import zipfile
from django.core.files import File

# Misc.
import re
from django.conf import settings
from django.utils.timezone import now

# Commands
from django.core.management.base import CommandError
from calaccess_raw.management.commands import CalAccessCommand
from calaccess_raw.models.tracking import RawDataVersion, RawDataFile


class Command(CalAccessCommand):
    """
    Extract the CAL-ACCESS raw data files from downloaded ZIP.
    """
    help = "Extract the CAL-ACCESS raw data files from downloaded ZIP"

    def add_arguments(self, parser):
        """
        Adds custom arguments specific to this command.
        """
        super(Command, self).add_arguments(parser)
        parser.add_argument(
            "--keep-files",
            action="store_true",
            dest="keep_files",
            default=False,
            help="Keep downloaded zipped files"
        )

    def handle(self, *args, **options):
        """
        Make it happen.
        """
        super(Command, self).handle(*args, **options)

        # Get the most recently downloaded RawDataVersion
        try:
            self.version = RawDataVersion.objects.latest_download()
        except RawDataVersion.DoesNotExist:
            raise CommandError('No record of a downloaded zip (run `python manage.py downloadcalaccessrawdata`).')

        # Raise exception if extract step did not finish
        if not self.version.download_completed:
            raise CommandError('Previous download did not finish (run `python manage.py downloadcalaccessrawdata`).')

        # Store extraction start time
        self.version.extract_start_datetime = now()
        # Reset the finish time
        self.version.extract_finish_datetime = None
        # Save in case the command doesn't finish
        self.version.save()

        self.header("Extracting raw data files")

        # flush tsv dir
        if os.path.exists(self.tsv_dir):
            shutil.rmtree(self.tsv_dir)
        os.mkdir(self.tsv_dir)

        self.extract_tsv_files()

        if getattr(settings, 'CALACCESS_STORE_ARCHIVE', False):
            self.archive()

        if not options['keep_files']:
            shutil.rmtree(self.download_dir)

        # store extraction finish time
        self.version.extract_finish_datetime = now()

        # and save the RawDataVersion
        self.version.save()

    def extract_tsv_files(self):
        """
        Extract all files with .TSV extension from downloaded zip.
        """
        if self.verbosity:
            self.log(" Extracting .TSV files")
        pattern = r'^.+\.TSV$'
        with zipfile.ZipFile(self.zip_path) as zf:
            tsv_files = [f for f in zf.namelist() if re.match(pattern, f)]
            for f in tsv_files:
                # extract
                extracted_path = zf.extract(f, self.download_dir)
                # move
                file_name = os.path.basename(extracted_path).upper()
                shutil.move(
                    extracted_path,
                    os.path.join(self.tsv_dir, file_name),
                )
                # track
                self.track_file(file_name)

    def track_file(self, file_name):
        """
        Create a RawDataFile object for for each download .TSV file.
        """
        raw_file, created = RawDataFile.objects.get_or_create(
            version=self.version,
            file_name=file_name.replace('.TSV', '')
        )
        # if raw file was already there, clear out timestamp fields
        if not created:
            raw_file.clean_start_datetime = None
            raw_file.clean_finish_datetime = None
            raw_file.load_start_datetime = None
            raw_file.load_finish_datetime = None
            raw_file.save()

    def archive(self):
        """
        Save a copy of the download zip file and each file inside.
        """
        if self.verbosity:
            self.log(" Archiving original files")
        if self.verbosity > 2:
            self.log(" Archiving {}".format(os.path.basename(self.zip_path)))

        for raw_file in self.version.files.all():
            if self.verbosity > 2:
                self.log(" Archiving {0}.TSV".format(raw_file.file_name))
            # Remove previous .TSV file
            raw_file.download_file_archive.delete()
            # Open up the .TSV file so we can wrap it in the Django File obj
            file_path = os.path.join(self.tsv_dir, raw_file.file_name + '.TSV')
            with open(file_path, 'rb') as f:
                # Save the .TSV on the raw data file
                raw_file.download_file_archive.save(
                    raw_file.file_name + '.TSV',
                    File(f)
                )
        return



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from setuptools import setup

def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()

setup(
    name='django-calaccess-raw-data',
    version='2.0.0',
    author='California Civic Data Coalition',
    author_email='cacivicdata@gmail.com',
    url='http://django-calaccess.californiacivicdata.org/',
    description=("A Django app to download, extract and load campaign "
                "finance and lobbying activity data from the California "
                "Secretary of State's CAL-ACCESS database"),
    long_description=read('README.rst'),
    license='MIT',
    packages=(
        'calaccess_raw',
        'calaccess_raw.admin',
        'calaccess_raw.management',
        'calaccess_raw.migrations',
        'calaccess_raw.management.commands',
        'calaccess_raw.models',
        'calaccess_raw.annotations'
    ),
    include_package_data=True,
    zip_safe=False,
    install_requires=(
        'django>=1.9',
        'django-postgres-copy>=2.3.5',
        'csvkit>=1.0',
        'requests',
        'clint',
        'hurry.filesize',
        'pytz'
    ),
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'License :: OSI Approved :: MIT License'
    ),
    project_urls={
        'Project': 'https://www.californiacivicdata.org/',
        'Documentation': 'http://django-calaccess.californiacivicdata.org',
        'Funding': 'https://www.californiacivicdata.org/about/',
        'Source': 'https://github.com/california-civic-data-coalition/django-calaccess-raw-data',
        'Coverage': 'https://coveralls.io/github/california-civic-data-coalition/django-calaccess-raw-data?branch=master',
        'Tracker': 'https://github.com/california-civic-data-coalition/django-calaccess-raw-data/issues'
    },
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-10-28 01:29
from __future__ import unicode_literals

import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0010_auto_20161026_1643'),
    ]

    operations = [
        migrations.AlterField(
            model_name='expncd',
            name='bakref_tid',
            field=calaccess_raw.fields.CharField(blank=True, db_column='BAKREF_TID', help_text="Back Reference to a Tran_ID of a 'parent' record", max_length=20, verbose_name='back reference transaction id'),
        ),
        migrations.AlterField(
            model_name='expncd',
            name='bal_juris',
            field=calaccess_raw.fields.CharField(blank=True, db_column='BAL_JURIS', help_text="Ballot measure's jurisdiction", max_length=40, verbose_name='ballot measure jurisdiction'),
        ),
        migrations.AlterField(
            model_name='expncd',
            name='bal_name',
            field=calaccess_raw.fields.CharField(blank=True, db_column='BAL_NAME', help_text='Ballot Measure Name', max_length=200, verbose_name='ballot measure name'),
        ),
        migrations.AlterField(
            model_name='expncd',
            name='bal_num',
            field=calaccess_raw.fields.CharField(blank=True, db_column='BAL_NUM', help_text='Ballot Number or Letter', max_length=7, verbose_name='ballot measure number'),
        ),
        migrations.AlterField(
            model_name='expncd',
            name='expn_dscr',
            field=calaccess_raw.fields.CharField(blank=True, db_column='EXPN_DSCR', help_text='Purpose of expense and/or description/explanation', max_length=400, verbose_name='expense description'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-04-21 18:21
from __future__ import unicode_literals

import calaccess_raw.annotations
import calaccess_raw.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('calaccess_raw', '0013_auto_20161123_2219'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rawdatafile',
            options={'ordering': ('-version_id', 'file_name'), 'verbose_name': 'TRACKING: CAL-ACCESS raw data file'},
        ),
        migrations.AlterModelOptions(
            name='rawdataversion',
            options={'get_latest_by': 'release_datetime', 'ordering': ('-release_datetime',), 'verbose_name': 'TRACKING: CAL-ACCESS raw data version'},
        ),
        migrations.AlterField(
            model_name='rcptcd',
            name='tran_type',
            field=calaccess_raw.fields.CharField(blank=True, choices=[('F', 'Forgiven Loan'), ('I', 'Intermediary'), ('R', 'Returned'), ('T', 'Third Party Repayment'), ('X', 'Transfer'), ('0', 'Unknown'), ('M', 'Unknown'), ('N', 'Unknown')], db_column='TRAN_TYPE', documentcloud_pages=[calaccess_raw.annotations.DocumentCloud(id='2711616-MapCalFormat2Fields', start_page=72), calaccess_raw.annotations.DocumentCloud(id='2712033-Cal-Format-1-05-02', start_page=29), calaccess_raw.annotations.DocumentCloud(id='2712034-Cal-Format-201', start_page=38)], help_text='Transaction Type', max_length=1, verbose_name='transaction type'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Utilities for representing and interacting with CAL-ACCESS reference documents and forms.
"""
from __future__ import unicode_literals

# Helpers
import os
import csvkit

# Annotations data
from . import choices
from .forms import FilingForm, FilingFormSection
from .documents import DocumentCloud, DocumentCloudPage


def load_forms():
    """
    Load all the FilingForm objects from the source CSV.
    """
    this_dir = os.path.dirname(__file__)

    # Read in forms
    form_path = os.path.join(this_dir, 'forms.csv')
    with open(form_path, 'r') as form_obj:
        form_reader = csvkit.DictReader(form_obj)
        form_list = [FilingForm(**row) for row in form_reader]

    # Read in sections
    section_path = os.path.join(this_dir, 'sections.csv')
    with open(section_path, 'r') as section_obj:
        section_reader = csvkit.DictReader(section_obj)
        for section in section_reader:
            form = next((x for x in form_list if x.id == section['form_id']))
            form.add_section(**section)

    # Pass it out
    return form_list


# Boot up all the forms from our source CSV files
FORMS = load_forms()


def get_form(id):
    """
    Takes an id for a filing form and returns a FilingForm object.
    """
    return next((x for x in FORMS if x.id == id.upper()), None)


def sort_choices(codes_dict):
    """
    Returns a tuple of tuples, sorted by the given codes_dict's key.
    """
    return tuple(sorted(codes_dict.items(), key=lambda x: x[0]))


__all__ = (
    "DocumentCloud",
    "DocumentCloudPage",
    "FilingForm",
    "FilingFormSection",
    'FORMS',
    'get_form',
    'choices',
    "sort_choices"
)
